@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"encoding/json"
 	"encoding/xml"
+	"errors"
 	"fmt"
 	"github.com/PuerkitoBio/goquery"
 	"github.com/antchfx/xmlquery"
@@ -143,6 +144,9 @@ func FormatXml(reader io.Reader, writer io.Writer, indent string, colors int) er
 				_, _ = fmt.Fprint(writer, tagColor(">"))
 				startTagClosed = true
 			}
+			if hasContent && (strings.Contains(str, "&") || strings.Contains(str, "<")) {
+				str = "<![CDATA[" + str + "]]>"
+			}
 			_, _ = fmt.Fprint(writer, str)
 		case xml.Comment:
 			if !startTagClosed {
@@ -183,6 +187,9 @@ func FormatXml(reader io.Reader, writer io.Writer, indent string, colors int) er
 			}
 			hasContent = false
 			lastTagName = currentTagName
+			if startTagClosed {
+				lastTagName = ""
+			}
 		case xml.Directive:
 			_, _ = fmt.Fprint(writer, tagColor("<!"), string(typedToken), tagColor(">"))
 			_, _ = fmt.Fprint(writer, newline, strings.Repeat(indent, level))
@@ -224,6 +231,10 @@ func XPathQuery(reader io.Reader, writer io.Writer, query string, singleNode boo
 		}
 	} else {
 		expr, _ := xpath.Compile(query)
+		if expr == nil {
+			return errors.New("unable to parse the XPath query")
+		}
+
 		val := expr.Evaluate(xmlquery.CreateXPathNavigator(doc))
 
 		switch typedVal := val.(type) {
